unit RsButtonDrawUtils;

{
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in compliance
  with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/MPL-1.1.html

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
  the specific language governing rights and limitations under the License.

  Copyright (c) 2012  .. (UA3GDW)
  E-mail: skl@griazy.lipetsk.ru, ua3gdw@radiosoft.info
  URL: http://radiosoft.info
}

interface

uses
  Windows, Types, Classes, Graphics, ImgList, Buttons;

type
  { TDrawUtils }

  TDrawUtils = class
  private
    FBitmap: TBitmap;
    FIndex: Integer;
    FImageList: TCustomImageList;
    FOnChange: TNotifyEvent;
    procedure BitmapChanged(Sender: TObject);
    procedure AddImageList;
    procedure SetBitmap(Value: TBitmap);
    procedure DrawButtonText(ACanvas: TCanvas; const ACaption: String;
      ATextBounds: TRect; AState: TButtonState; ABiDiFlags: Longint);
    procedure CalcButtonLayout(ACanvas: TCanvas; const AClient: TRect;
      const AOffset: TPoint; const ACaption: String; ALayout: TButtonLayout;
      AMargin, ASpacing: Integer; var ABitmapPos: TPoint; var ATextBounds: TRect;
      ABiDiFlags: LongInt);
    procedure DrawButtonBitmap(ACanvas: TCanvas; const ABitmapPos: TPoint;
      AState: TButtonState; ATransparent: Boolean; AEnabled: Boolean);
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Draw(ACanvas: TCanvas; const AClient: TRect; const AOffset: TPoint;
      const ACaption: String; ALayout: TButtonLayout; AMargin, ASpacing: Integer;
      AState: TButtonState; ATransparent: Boolean; ABiDiFlags: LongInt;
      AEnabled: Boolean);
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  procedure DrawIconShadow(ACanvas: TCanvas; const ARect: TRect;
    ImageList: TCustomImageList; ImageIndex: Integer);

implementation

uses
  Themes;

procedure DrawIconShadow(ACanvas: TCanvas; const ARect: TRect;
  ImageList: TCustomImageList; ImageIndex: Integer);
var
  ImageWidth, ImageHeight: Integer;
  I, J                   : Integer;
  Src, Dst               : ^Cardinal;
  S, C, CBRB, CBG        : Cardinal;
  B1, B2                 : TBitmap;
begin
  ImageWidth := RectWidth(ARect);
  ImageHeight := RectHeight(ARect);

  with ImageList do
    begin
      if Width < ImageWidth then
        ImageWidth := Width;
      if Height < ImageHeight then
        ImageHeight :=  Height;
    end;

  B1 := TBitmap.Create;
  B2 := TBitmap.Create;

  try
    with B1 do
      begin
        PixelFormat := pf32bit;
        Width := ImageWidth;
        Height := ImageHeight;
      end;

    with B2 do
      begin
        PixelFormat := pf32bit;
        Width := ImageWidth;
        Height := ImageHeight;
      end;

    BitBlt(B1.Canvas.Handle, 0, 0, ImageWidth, ImageHeight, ACanvas.Handle,
      ARect.Left, ARect.Top, SRCCOPY);
    BitBlt(B2.Canvas.Handle, 0, 0, ImageWidth, ImageHeight, ACanvas.Handle,
      ARect.Left, ARect.Top, SRCCOPY);

    ImageList.Draw(B2.Canvas, 0, 0, ImageIndex, True);

    for J := 0 to ImageHeight - 1 do
      begin
        Src := B2.ScanLine[J];
        Dst := B1.ScanLine[J];

        for I := 0 to ImageWidth - 1 do
          begin
            S := Src^;

            if S <> Dst^ then
              begin
                CBRB := Dst^ and $00FF00FF;
                CBG  := Dst^ and $0000FF00;
                C := ((S and $00FF0000) shr 16 * 29 + (S and $0000FF00) shr 8 * 150 +
                  (S and $000000FF) * 76) shr 8;
                C := (C div 3) + (255 - 255 div 3);
                Dst^ := ((CBRB * C and $FF00FF00) or (CBG * C and $00FF0000)) shr 8;
              end;
            Inc(Src);
            Inc(Dst);
          end;
      end;

    BitBlt(ACanvas.Handle, ARect.Left, ARect.Top, ImageWidth, ImageHeight,
      B1.Canvas.Handle, 0, 0, SRCCOPY);
  finally
    B1.Free;
    B2.Free;
  end;
end;

{ TDrawUtils }

{*******************************************************************************
Procedure  :  CalcButtonLayout
Description:        
               .    var - .
Parameters :  ACanvas     - 
              AClient     -  
              AOffset     -  
              ACaption    - 
              ALayout     -  
              AMargin     -      ( -1, 
                             )
              ABitmapPos  -  
              ATextBounds -  
              ABiDiFlags  -  - 
*******************************************************************************}
procedure TDrawUtils.CalcButtonLayout(ACanvas: TCanvas; const AClient: TRect;
  const AOffset: TPoint; const ACaption: String; ALayout: TButtonLayout;
  AMargin, ASpacing: Integer; var ABitmapPos: TPoint; var ATextBounds: TRect;
  ABiDiFlags: Integer);
var
  TextPos   : TPoint;
  ClientSize: TPoint;
  GlyphSize : TPoint;
  TextSize  : TPoint;
  TotalSize : TPoint;
begin
  if (ABiDiFlags and DT_RIGHT) = DT_RIGHT then
    if ALayout = blGlyphLeft then
      ALayout := blGlyphRight
    else
      if ALayout = blGlyphRight then
        ALayout := blGlyphLeft;

  ClientSize := Point(AClient.Right - AClient.Left,
    AClient.Bottom - AClient.Top);
    
  if FBitmap <> nil then
    GlyphSize := Point(FBitmap.Width, FBitmap.Height)
  else
    GlyphSize := Point(0, 0);
    
  if Length(ACaption) > 0 then
    begin
      ATextBounds := Rect(0, 0, AClient.Right - AClient.Left, 0);
      DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ATextBounds,
        DT_CALCRECT or ABiDiFlags);
      TextSize := Point(ATextBounds.Right - ATextBounds.Left,
        ATextBounds.Bottom - ATextBounds.Top);
    end
  else
    begin
      ATextBounds := Rect(0, 0, 0, 0);
      TextSize := Point(0,0);
    end;
    
  {       ,
        .
         , 
        }
  if ALayout in [blGlyphLeft, blGlyphRight] then
    begin
      ABitmapPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
      TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
    end
  else
    begin
      ABitmapPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
      TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
    end;

  {     ,    0 }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then ASpacing := 0;

  if AMargin = -1 then
    begin
      if ASpacing = -1 then
        begin
          TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
          if ALayout in [blGlyphLeft, blGlyphRight] then
            AMargin := (ClientSize.X - TotalSize.X) div 3
          else
            AMargin := (ClientSize.Y - TotalSize.Y) div 3;
          ASpacing := AMargin;
        end
      else
        begin
          TotalSize := Point(GlyphSize.X + ASpacing + TextSize.X, GlyphSize.Y +
            ASpacing + TextSize.Y);
          if ALayout in [blGlyphLeft, blGlyphRight] then
            AMargin := (ClientSize.X - TotalSize.X + 1) div 2
          else
            AMargin := (ClientSize.Y - TotalSize.Y + 1) div 2;
        end;
    end
  else
    begin
      if ASpacing = -1 then
        begin
          TotalSize := Point(ClientSize.X - (AMargin + GlyphSize.X),
                             ClientSize.Y - (AMargin + GlyphSize.Y));
          if ALayout in [blGlyphLeft, blGlyphRight] then
            ASpacing := (TotalSize.X - TextSize.X) div 2
          else
            ASpacing := (TotalSize.Y - TextSize.Y) div 2;
        end;
    end;

  with ABitmapPos do
    begin
      case ALayout of
        blGlyphLeft:
          begin
            X := AMargin;
            TextPos.X := X + GlyphSize.X + ASpacing;
          end;
        blGlyphRight:
          begin
            X := ClientSize.X - AMargin - GlyphSize.X;
            TextPos.X := X - ASpacing - TextSize.X;
          end;
        blGlyphTop:
          begin
            Y := AMargin;
            TextPos.Y := Y + GlyphSize.Y + ASpacing;
          end;
        blGlyphBottom:
          begin
            Y := ClientSize.Y - AMargin - GlyphSize.Y;
            TextPos.Y := Y - ASpacing - TextSize.Y;
          end;
      end;

      Inc(X, AClient.Left + AOffset.X);
      Inc(Y, AClient.Top + AOffset.Y);
    end;

  if StyleServices.Enabled then
    {   XP  }
    OffsetRect(ATextBounds, TextPos.X + AClient.Left, TextPos.Y + AClient.Top)
  else
    OffsetRect(ATextBounds, TextPos.X + AClient.Left + AOffset.X,
      TextPos.Y + AClient.Top + AOffset.Y);
end;

constructor TDrawUtils.Create;
begin
  FBitmap := TBitmap.Create;
  FBitmap.OnChange := BitmapChanged;
  FIndex := -1;
end;

destructor TDrawUtils.Destroy;
begin
  if FImageList <> nil then
    FImageList.Free;
  FBitmap.Free;
  inherited Destroy;
end;

{*******************************************************************************
Procedure  :  Draw
Description:       Canvas .
Parameters :  ACanvas      - 
              AClient      -   
              AOffset      -  
              ASpacing     -    
              AState       -  
              ATransparent -  
              ABiDiFlags   -  - 
              AEnabled     -  
*******************************************************************************}
procedure TDrawUtils.Draw(ACanvas: TCanvas; const AClient: TRect;
  const AOffset: TPoint; const ACaption: String; ALayout: TButtonLayout;
  AMargin, ASpacing: Integer; AState: TButtonState; ATransparent: Boolean;
  ABiDiFlags: Integer; AEnabled: Boolean);
var
  GlyphPos: TPoint;
  R       : TRect;
begin
  CalcButtonLayout(ACanvas, AClient, AOffset, ACaption, ALayout, AMargin,
    ASpacing, GlyphPos, R, ABiDiFlags);
  DrawButtonBitmap(ACanvas, GlyphPos, AState, ATransparent, AEnabled);
  OffsetRect(R, AOffset.X, AOffset.Y);
  DrawButtonText(ACanvas, ACaption, R, AState, ABiDiFlags);
end;

{*******************************************************************************
Procedure  :  DrawButtonBitmap
Description:     Canvas .
Parameters :  ACanvas      - 
              AGlyphPos    -  
              AState       -  
              ATransparent - 
              AEnabled     -  
*******************************************************************************}
procedure TDrawUtils.DrawButtonBitmap(ACanvas: TCanvas; const ABitmapPos: TPoint;
  AState: TButtonState; ATransparent: Boolean; AEnabled: Boolean);
begin
  if FIndex <> -1 then
    with ABitmapPos do
      begin
        if StyleServices.Enabled then
          {  XP  }
          FImageList.Draw(ACanvas, X, Y, 0, dsNormal, itImage, AEnabled)
        else
          {  XP   }
          if ATransparent or (AState = bsExclusive) then
            FImageList.Draw(ACanvas, X, Y, 0, dsTransparent, itImage, AEnabled)
          else
            FImageList.Draw(ACanvas, X, Y, 0, dsNormal, itImage, AEnabled);
      end;
end;

{*******************************************************************************
Procedure  :  DrawButtonText
Description:     Canvas .
Parameters :  ACanvas     - 
              ACaption    - 
              ATextBounds -   
              AState      -  
              ABiDiFlags  -  - 
*******************************************************************************}
procedure TDrawUtils.DrawButtonText(ACanvas: TCanvas; const ACaption: String;
  ATextBounds: TRect; AState: TButtonState; ABiDiFlags: Integer);
var
  iLenCaption: Integer;
begin
  iLenCaption := Length(ACaption);
  with ACanvas do
    begin
      Brush.Style := bsClear;
      if AState = bsDisabled then
        begin
          OffsetRect(ATextBounds, 1, 1);
          Font.Color := clBtnHighlight;
          DrawText(Handle, PChar(ACaption), iLenCaption, ATextBounds,
            DT_CENTER or DT_VCENTER or ABiDiFlags);
          OffsetRect(ATextBounds, -1, -1);
          Font.Color := clBtnShadow;
          DrawText(Handle, PChar(ACaption), iLenCaption, ATextBounds,
            DT_CENTER or DT_VCENTER or ABiDiFlags);
        end
      else
        DrawText(Handle, PChar(ACaption), iLenCaption, ATextBounds,
          DT_CENTER or DT_VCENTER or ABiDiFlags);
    end;
end;

{*******************************************************************************
Procedure  :  SetBitmap
Description:     FBitmap,  
              ImageList     
Parameters :  Value - 
*******************************************************************************}
procedure TDrawUtils.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
  AddImageList;
end;

procedure TDrawUtils.BitmapChanged(Sender: TObject);
begin
  AddImageList;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TDrawUtils.AddImageList;
var
  BitMask: TBitmap;
begin
  FIndex := -1;

  {  ImageList  ,    }
  if FImageList <> nil then
    begin
      FImageList.Free;
      FImageList := nil;
    end;

  if (FBitmap <> nil) and (FBitmap.Height > 0) then
    begin
      {  ImageList    }
      FImageList := TCustomImageList.CreateSize(FBitmap.Width, FBitmap.Height);

      {   }
      BitMask := TBitmap.Create;
      with BitMask do
        begin
          Assign(FBitmap);
          Mask(FBitmap.Canvas.Pixels[0, 0]);
        end;

      {      ImageList
          }
      FIndex := FImageList.Add(FBitmap, BitMask);
      //***

      BitMask.Free;
  end;
end;

end.
