unit RsButtons;

{
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in compliance
  with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/MPL-1.1.html

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
  the specific language governing rights and limitations under the License.

  Copyright (c) 2012  .. (UA3GDW)
  E-mail: skl@griazy.lipetsk.ru, ua3gdw@radiosoft.info
  URL: http://radiosoft.info
}

interface

uses
  Windows, SysUtils, Classes, Controls, StdCtrls, ImgList, CommCtrl, VCL.Buttons,
  Graphics, Messages, VCL.ActnList, RsButtonDrawUtils;

type
  TRsBitBtn = class(TBitBtn)
  private
    FBitmap: TBitmap;
    FImages: TCustomImageList;
    FImageIndex: TImageIndex;
    FImageChangeLink: TChangeLink;
    FOnPosChanged: TNotifyEvent;
    FAllowDrawFocus: Boolean;

    procedure BitmapChanged(Sender: TObject);
    procedure ImageListChange(Sender: TObject);
    procedure SetBitmap(Value: TBitmap);
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged);
      message WM_WINDOWPOSCHANGED;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetImages(Value: TCustomImageList);
    procedure MakeBitmap(ABitmap: TBitmap);
    procedure SetAllowDrawFocus(const Value: Boolean);
    function GetGlyph: TBitmap;
    procedure SetGlyph(const Value: TBitmap);
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Images: TCustomImageList read FImages write SetImages;
    property ImageIndex: TImageIndex read FImageIndex
      write SetImageIndex default -1;
    property OnPosChanged: TNotifyEvent read FOnPosChanged
      write FOnPosChanged;
    property AllowDrawFocus: Boolean read FAllowDrawFocus
      write SetAllowDrawFocus default True;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
  end;

  { TRsSpeedButton }
  TRsSpeedButton = class(TSpeedButton)
  private
    FBitmap: TBitmap;
    FImages: TCustomImageList;
    FImageIndex: TImageIndex;
    FImageChangeLink: TChangeLink;
    FThemeEnabled: Boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FAutoSize: Boolean;
    FAllowHeightAutoSize: Boolean;
    FOnResize: TNotifyEvent;
    FVisibleChanged: TNotifyEvent;
    FMouseInControl: Boolean;
    //FColor: TColor;

    procedure BitmapChanged(Sender: TObject);
    procedure SetBitmap(Value: TBitmap);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetImages(Value: TCustomImageList);
    procedure ImageListChange(Sender: TObject);
    procedure SetThemeEnabled(const Value: Boolean);
    procedure DoMouseEnter;
    procedure DoMouseLeave;
    procedure DoResize;
    procedure SetAllowHeightAutoSize(const Value: Boolean);
    procedure MakeBitmap(ABitmap: TBitmap);
    function GetGlyph: TBitmap;
    procedure SetGlyph(const Value: TBitmap);
    //procedure SetColor(const Value: TColor);
  protected
    procedure WndProc(var Msg: TMessage); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure SetAutoSize(const Value: Boolean); reintroduce;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Resize; override;
    property OnVisibleChanged: TNotifyEvent read FVisibleChanged
      write FVisibleChanged;
  published
    property Images: TCustomImageList read FImages write SetImages;
    property ImageIndex: TImageIndex read FImageIndex
      write SetImageIndex default -1;
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property ThemeEnabled: Boolean read FThemeEnabled
      write SetThemeEnabled default True;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property AutoSize: Boolean read FAutoSize write SetAutoSize default False;
    property AllowHeightAutoSize: Boolean read FAllowHeightAutoSize
      write SetAllowHeightAutoSize default False;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    //property Color: TColor read FColor write SetColor default clBtnFace;
  end;

implementation

uses
  Types, Math;

{ TRsBitBtn }

procedure TRsBitBtn.CNDrawItem(var Message: TWMDrawItem);
begin
  if not FAllowDrawFocus then
    with Message do
      DrawItemStruct.itemState := DrawItemStruct.itemState - ODS_FOCUS;
  inherited;
end;

constructor TRsBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBitmap := TBitmap.Create;
  FBitmap.OnChange := BitmapChanged;
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  FImageIndex := -1;
  FAllowDrawFocus := True;
end;

destructor TRsBitBtn.Destroy;
begin
  FBitmap.Free;
  FImageChangeLink.Free;
  inherited Destroy;
end;

function TRsBitBtn.GetGlyph: TBitmap;
begin
  Result := inherited Glyph;
end;

procedure TRsBitBtn.SetAllowDrawFocus(const Value: Boolean);
begin
  if FAllowDrawFocus <> Value then
    begin
      FAllowDrawFocus := Value;
      Invalidate;
    end;
end;

procedure TRsBitBtn.SetBitmap(Value: TBitmap);
begin
  {  Bitmap  ,  TListImage
       ImageIndex = 0. }
  if (FImages = nil) or ((FImages <> nil) and (FImageIndex = -1)) then
    FBitmap.Assign(Value);
end;

procedure TRsBitBtn.SetGlyph(const Value: TBitmap);
begin
  if FBitmap.Handle = 0 then
    inherited Glyph := Value;
end;

procedure TRsBitBtn.WMWindowPosChanged(var Message: TWMWindowPosChanged);
begin
  inherited;
  if Assigned(FOnPosChanged) then
    FOnPosChanged(Self);
end;

procedure TRsBitBtn.BitmapChanged(Sender: TObject);
begin
  {  Bitmap  ,  TListImage
       ImageIndex = 0. }
  if (FImages = nil) or ((FImages <> nil) and (FImageIndex = -1)) then
    MakeBitmap(TBitmap(Sender));
end;

procedure TRsBitBtn.SetImages(Value: TCustomImageList);
begin
  FImages := Value;

  if FImages <> nil then
    with FImages do
      begin
        UnRegisterChanges(FImageChangeLink);
        RegisterChanges(FImageChangeLink);
        FreeNotification(Self);

        if FImageIndex <> -1 then
          begin
            FBitmap.Handle := 0;
            GetBitmap(FImageIndex, FBitmap);
            MakeBitmap(FBitmap);
          end;
      end
  else
    begin
      if FImageIndex <> -1 then
        begin
          FImageIndex := -1;
          FBitmap.Handle := 0;
          MakeBitmap(nil);
        end;
    end;
end;

procedure TRsBitBtn.ImageListChange(Sender: TObject);
begin
  FBitmap.Handle := 0;
  FImages.GetBitmap(FImageIndex, FBitmap);
  MakeBitmap(FBitmap);
end;

procedure TRsBitBtn.MakeBitmap(ABitmap: TBitmap);
var
  Bmp    : TBitmap;
  ImgList: TImageList;
begin
  if (ABitmap <> nil) and (ABitmap.Width > 0) and (ABitmap.Height > 0) then
    begin
      Bmp := TBitmap.Create;
      ImgList := TImageList.CreateSize(ABitmap.Width, ABitmap.Height);

      try
        ImgList.Add(ABitmap, nil);

        with Bmp do
          begin
            Width := FBitmap.Width * 2;
            Height := FBitmap.Height;

            {  . }
            Canvas.Draw(0, 0, FBitmap);
          end;

        {  . }
        DrawIconShadow(Bmp.Canvas, Rect(ABitmap.Width, 0, Bmp.Width,
          Bmp.Height), ImgList, 0);

        inherited Glyph := Bmp;
        NumGlyphs := 2;
        Invalidate;
      finally
        Bmp.Free;
        ImgList.Free;
      end;
    end
  else
    inherited Glyph := nil;

  //Resize;
  Invalidate;
end;

procedure TRsBitBtn.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FImages) then
    begin
      if FImageIndex <> -1 then
        begin
          FBitmap.Handle := 0;
          MakeBitmap(nil);
        end;

      FImageIndex := -1;
      FImages := nil;
    end;
end;

procedure TRsBitBtn.SetImageIndex(const Value: TImageIndex);
begin
  if FImageIndex <> Value then
    begin
      FImageIndex := Value;

      if FImages <> nil then
        begin
          if FImageIndex <> -1 then
            begin
              FBitmap.Handle := 0;
              FImages.GetBitmap(FImageIndex, FBitmap);
              MakeBitmap(FBitmap);
            end
          else
            begin
              FBitmap.Handle := 0;
              MakeBitmap(nil);
            end;
        end;
    end;
end;

procedure TRsBitBtn.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited ActionChange(Sender, CheckDefaults);
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
      begin
        if not CheckDefaults or (Self.ImageIndex = -1) then
          Self.ImageIndex := TCustomAction(Sender).ImageIndex;

        if FBitmap.Empty and (ActionList <> nil) and (ActionList.Images <> nil)
          and (ImageIndex >= 0) and (ImageIndex < ActionList.Images.Count) then
            begin
              FBitmap.Handle := 0;
              ActionList.Images.GetBitmap(ImageIndex, FBitmap);
              MakeBitmap(FBitmap);
            end;
      end;
end;

{ TRsSpeedButton }

procedure TRsSpeedButton.CMMouseEnter(var Message: TMessage);
var
  bNeedRepaint: Boolean;
begin
  inherited;

  bNeedRepaint := not FMouseInControl and Enabled and
    (DragMode <> dmAutomatic) and (GetCapture = 0);

  if bNeedRepaint and not (csDesigning in ComponentState) then
    begin
      FMouseInControl := True;
      MouseCapture := True;
      if not FThemeEnabled then
        Invalidate;
      DoMouseEnter;
    end;
end;

procedure TRsSpeedButton.CMMouseLeave(var Message: TMessage);
var
  bNeedRepaint: Boolean;
begin
  inherited;
  bNeedRepaint := FMouseInControl and Enabled and not Dragging;

  if bNeedRepaint and not (csDesigning in ComponentState) then
    begin
      FMouseInControl := False;
      MouseCapture := False;
      if not FThemeEnabled then
        Invalidate;
      DoMouseLeave;
    end;
end;

constructor TRsSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBitmap := TBitmap.Create;
  FBitmap.OnChange := BitmapChanged;
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  FImageIndex := -1;
  FThemeEnabled := True;
//  FColor := clBtnFace;
end;

destructor TRsSpeedButton.Destroy;
begin
  FBitmap.Free;
  FImageChangeLink.Free;
  inherited Destroy;
end;

procedure TRsSpeedButton.DoMouseEnter;
begin
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(Self);
end;

procedure TRsSpeedButton.DoMouseLeave;
begin
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(Self);
end;

procedure TRsSpeedButton.DoResize;
begin
  if Assigned(FOnResize) then
    FOnResize(Self);
end;

function TRsSpeedButton.GetGlyph: TBitmap;
begin
  Result := inherited Glyph;
end;

procedure TRsSpeedButton.Resize;
begin
  inherited;
  if FAutoSize then
    begin
      Width := FBitmap.Width + Canvas.TextWidth(Caption) + Spacing +
        10 + Margin;

      if FAllowHeightAutoSize then
        Height := Max(FBitmap.Height + 10,
          Canvas.TextHeight(Caption) + 10);
    end;
  DoResize;
end;

procedure TRsSpeedButton.SetBitmap(Value: TBitmap);
begin
  {  Bitmap  ,  TListImage
       ImageIndex = 0. }
  if (FImages = nil) or ((FImages <> nil) and (FImageIndex = -1)) then
    FBitmap.Assign(Value);
end;

//procedure TRsSpeedButton.SetColor(const Value: TColor);
//begin
//  if FColor <> Value then
//    begin
//      FColor := Value;
//      Invalidate;
//    end;
//end;

procedure TRsSpeedButton.SetGlyph(const Value: TBitmap);
begin
  if FBitmap.Handle = 0 then
    inherited Glyph := Value;
end;

function TRsSpeedButton.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result := inherited;

  if FAutoSize then
    Resize
  else
    DoResize;
end;

procedure TRsSpeedButton.SetAllowHeightAutoSize(const Value: Boolean);
begin
  if FAllowHeightAutoSize <> Value then
    begin
      FAllowHeightAutoSize := Value;
      Resize;
    end;
end;

procedure TRsSpeedButton.SetAutoSize(const Value: Boolean);
begin
  if FAutoSize <> Value then
    begin
      FAutoSize := Value;
      Resize;
    end;
end;

procedure TRsSpeedButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited ActionChange(Sender, CheckDefaults);
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
      begin
        if CheckDefaults or (Self.GroupIndex = 0) then
          Self.GroupIndex := GroupIndex;

        if not CheckDefaults or (Self.ImageIndex = -1) then
          Self.ImageIndex := TCustomAction(Sender).ImageIndex;

        if FBitmap.Empty and (ActionList <> nil) and (ActionList.Images <> nil)
          and (ImageIndex >= 0) and (ImageIndex < ActionList.Images.Count) then
            begin
              FBitmap.Handle := 0;
              ActionList.Images.GetBitmap(ImageIndex, FBitmap);
              MakeBitmap(FBitmap);
            end;
      end;
end;

procedure TRsSpeedButton.BitmapChanged(Sender: TObject);
begin
  {  Bitmap  ,  TListImage
       ImageIndex = 0. }
  if (FImages = nil) or ((FImages <> nil) and (FImageIndex = -1)) then
    MakeBitmap(TBitmap(Sender));
end;

procedure TRsSpeedButton.WndProc(var Msg: TMessage);
var
  P           : TPoint;
  bNeedRepaint: Boolean;
begin
  inherited;

  case Msg.Msg of
    WM_MOUSEMOVE, WM_NCMOUSEMOVE:
      begin
        P := Point(Msg.LParamLo, Msg.LParamHi); //  
        bNeedRepaint := FMouseInControl and Enabled and not Dragging;

        if not PtInRect(ClientRect, P) and bNeedRepaint then
          begin //  CM_MOUSELEAVE
            MouseCapture := False;
            FMouseInControl := False;
            DoMouseLeave;
            if not FThemeEnabled then
              Invalidate;
          end;
      end;
  end;  
end;

procedure TRsSpeedButton.SetImageIndex(const Value: TImageIndex);
begin
  if FImageIndex <> Value then
    begin
      FImageIndex := Value;

      if FImages <> nil then
        begin
          if FImageIndex <> -1 then
            begin
              FBitmap.Handle := 0;
              FImages.GetBitmap(FImageIndex, FBitmap);
              MakeBitmap(FBitmap);
            end
          else
            begin
              FBitmap.Handle := 0;
              MakeBitmap(nil);
            end;
        end;
    end;
end;

procedure TRsSpeedButton.SetImages(Value: TCustomImageList);
begin
  FImages := Value;

  if FImages <> nil then
    with FImages do
      begin
        UnRegisterChanges(FImageChangeLink);
        RegisterChanges(FImageChangeLink);
        FreeNotification(Self);

        if FImageIndex <> -1 then
          begin
            FBitmap.Handle := 0;
            GetBitmap(FImageIndex, FBitmap);
            MakeBitmap(FBitmap);
          end;
      end
  else
    begin
      if FImageIndex <> -1 then
        begin
          FImageIndex := -1;
          FBitmap.Handle := 0;
          MakeBitmap(nil);
        end;
    end;
end;

procedure TRsSpeedButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FImages) then
    begin
      if FImageIndex <> -1 then
        begin
          FBitmap.Handle := 0;
          MakeBitmap(nil);
        end;

      FImageIndex := -1;
      FImages := nil;
    end;
end;

procedure TRsSpeedButton.Paint;
const
  DownStyles: array[Boolean] of Integer = (BDR_RAISEDINNER, BDR_SUNKENOUTER);
  FillStyles: array[Boolean] of Integer = (BF_MIDDLE, 0);
var
  PaintRect : TRect;
  iDrawFlags: Integer;
  Offset    : TPoint;
  DrawUtils : TDrawUtils;
begin
  if Parent = nil then
    Exit;

  if FThemeEnabled then
    inherited
  else
    begin
      if not Enabled then
        FState := bsDisabled
      else if FState = bsDisabled then
        if Down and (GroupIndex <> 0) then
          FState := bsExclusive
        else
          FState := bsUp;

      Canvas.Font := Self.Font;
      PaintRect := Rect(0, 0, Width, Height);

      if not Flat then
        begin
          iDrawFlags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
          if FState in [bsDown, bsExclusive] then
            iDrawFlags := iDrawFlags or DFCS_PUSHED;
          DrawFrameControl(Canvas.Handle, PaintRect, DFC_BUTTON, iDrawFlags);
        end
      else
        begin
          if (FState in [bsDown, bsExclusive]) or (FMouseInControl and
            (FState <> bsDisabled)) or (csDesigning in ComponentState) then
            DrawEdge(Canvas.Handle, PaintRect,
              DownStyles[FState in [bsDown, bsExclusive]],
              FillStyles[Transparent] or BF_RECT);

          InflateRect(PaintRect, -1, -1);
        end;

      if FState in [bsDown, bsExclusive] then
        begin
          if (FState = bsExclusive) and (not Flat or not FMouseInControl) then
            begin
              Canvas.Brush.Bitmap := AllocPatternBitmap(clBtnFace, clBtnHighlight);
              Canvas.FillRect(PaintRect);
            end;
          Offset.X := 1;
          Offset.Y := 1;
        end
      else
        begin
          Offset.X := 0;
          Offset.Y := 0;
        end;

      DrawUtils := TDrawUtils.Create;
      try
        DrawUtils.Bitmap := FBitmap;
        DrawUtils.Draw(Canvas, PaintRect, Offset, Caption, Layout, Margin,
          Spacing, FState, Transparent, DrawTextBiDiModeFlags(0), Enabled);
      finally
        DrawUtils.Free;
      end;
    end;
end;

procedure TRsSpeedButton.ImageListChange(Sender: TObject);
begin
  FBitmap.Handle := 0;
  FImages.GetBitmap(FImageIndex, FBitmap);
  MakeBitmap(FBitmap);
end;

procedure TRsSpeedButton.MakeBitmap(ABitmap: TBitmap);
var
  Bmp    : TBitmap;
  ImgList: TImageList;
begin
  if (ABitmap <> nil) and (ABitmap.Width > 0) and (ABitmap.Height > 0) then
    begin
      Bmp := TBitmap.Create;
      ImgList := TImageList.CreateSize(ABitmap.Width, ABitmap.Height);

      try
        ImgList.Add(ABitmap, nil);

        with Bmp do
          begin
            Width := FBitmap.Width * 2;
            Height := FBitmap.Height;

            {  . }
            Canvas.Draw(0, 0, FBitmap);
          end;

        {  . }
        DrawIconShadow(Bmp.Canvas, Rect(ABitmap.Width, 0, Bmp.Width,
          Bmp.Height), ImgList, 0);

        inherited Glyph := Bmp;
        NumGlyphs := 2;
        Invalidate;
      finally
        Bmp.Free;
        ImgList.Free;
      end;
    end
  else
    inherited Glyph := nil;

  Resize;
  Invalidate;
end;

procedure TRsSpeedButton.SetThemeEnabled(const Value: Boolean);
begin
  if FThemeEnabled <> Value then
    begin
      FThemeEnabled := Value;
      Invalidate;
    end;
end;


end.

