unit RsColorPicker;

{
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in compliance
  with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/MPL-1.1.html

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
  the specific language governing rights and limitations under the License.

  Copyright (c) 2012  .. (UA3GDW)
  E-mail: skl@griazy.lipetsk.ru, ua3gdw@radiosoft.info
  URL: http://radiosoft.info
}

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, VCL.Controls, VCL.Graphics,
  RsPickerForm, Winapi.Messages, RsButtons, VCL.ExtCtrls, VCL.StdCtrls;

type
  TRsColorPicker = class(TCustomEdit)
  private
    { Private declarations }
    FColorButton: TRsBitBtn;
    FPickerForm: TPickerForm;
    FOpeningFromEnter: Boolean;
    FAllowNoneColor: Boolean;
    FCanvas: TCanvas;
    FOnSelectColor: TNotifyEvent;

    procedure ClickColorButtonEvent(Sender: TObject);
    function GetSelectedColor: TColor;
    procedure SetSelectedColor(const Value: TColor);
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure SetAllowNoneColor(const Value: Boolean);
    procedure WMSetCursor(var Msg: TWMSetCursor); message WM_SETCURSOR;
    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure WMContextMenu(var Message: TWMContextMenu); message WM_CONTEXTMENU;
    procedure AlignChangedEvent(Sender: TObject);
    procedure OnSelectColorEvent(Sender: TObject);
  protected
    { Protected declarations }
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property SelectedColor: TColor read GetSelectedColor
      write SetSelectedColor default clNone;
    property Visible;
    property Enabled;
    property TabOrder;
    property TabStop default True;
    property OpeningFromEnter: Boolean read FOpeningFromEnter
      write FOpeningFromEnter default True;
    property AllowNoneColor: Boolean read FAllowNoneColor
      write SetAllowNoneColor default True;
    property ColorButton: TRsBitBtn read FColorButton write FColorButton;
    property AutoSize;
    property OnSelectColor: TNotifyEvent read FOnSelectColor write FOnSelectColor;
    property PopupMenu;
    property Margins;
    property Align;
    property Anchors;
    property ShowHint;
    property ParentShowHint;
    property DoubleBuffered;
  end;

implementation

uses Forms, Types;

{ TRsPicker }

procedure TRsColorPicker.AlignChangedEvent(Sender: TObject);
begin
 Invalidate;
end;

procedure TRsColorPicker.ClickColorButtonEvent(Sender: TObject);
var
  P: TPoint;
begin
  SetFocus;
  Invalidate;

  P := Point(Left, Top + Height);

  if Parent is TWinControl then
    P := TWinControl(Parent).ClientToScreen(P);

  with FPickerForm do
    begin
      RsSpeedButton2.Visible := FAllowNoneColor;
      CreateColors;
    end;

  SetWindowPos(FPickerForm.Handle, HWND_TOPMOST, P.X, P.Y, 0, 0, SWP_NOSIZE or
    SWP_NOACTIVATE or SWP_SHOWWINDOW);

  with FPickerForm do
    begin
      Timer1.Enabled := True;
      IsShow := True;
    end;
end;

procedure TRsColorPicker.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  FColorButton.Enabled := Enabled;
end;

constructor TRsColorPicker.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle - [csDoubleClicks];
  ParentColor := False;
  TabStop := True;
  FAllowNoneColor := True;
  FCanvas := TCanvas.Create;
  FOpeningFromEnter := True;

  FColorButton := TRsBitBtn.Create(Self);
  with FColorButton do
    begin
      Name := ClassName;
      Parent := Self;
      Caption := '...';
      FreeNotification(Self);
      Align := alRight;
      Width := 19;
      OnClick := ClickColorButtonEvent;
      AllowDrawFocus := False;
      TabStop := False;
      SetSubComponent(True);
      OnPosChanged := AlignChangedEvent;
    end;

  FPickerForm := TPickerForm.Create(Self);
  FPickerForm.SelectedColor := clNone;
  FPickerForm.OnColorSelect := OnSelectColorEvent;

  if AOwner is TWinControl then
    FPickerForm.ParentForm := TWinControl(AOwner);
end;

destructor TRsColorPicker.Destroy;
begin
  FColorButton.Free;
  FPickerForm.Free;
  FCanvas.Free;
  inherited;
end;

function TRsColorPicker.GetSelectedColor: TColor;
begin
  Result := FPickerForm.SelectedColor;
end;

procedure TRsColorPicker.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if not FOpeningFromEnter or (csDesigning in ComponentState) then
    Exit;

  case Key of
    VK_RETURN:
      if Focused and not FPickerForm.IsShow then
        ClickColorButtonEvent(nil);
    VK_ESCAPE:
      if Focused and FPickerForm.IsShow then
        FPickerForm.Hide;
  end;
end;

procedure TRsColorPicker.KeyPress(var Key: Char);
begin
  Key := #0;
end;

procedure TRsColorPicker.Loaded;
begin
  inherited;
  Text := EmptyStr;
end;

procedure TRsColorPicker.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  SetFocus;
  Invalidate;
end;

procedure TRsColorPicker.OnSelectColorEvent(Sender: TObject);
begin
  if Assigned(FOnSelectColor) then
    FOnSelectColor(Self);
end;

procedure TRsColorPicker.SetAllowNoneColor(const Value: Boolean);
begin
  FAllowNoneColor := Value;
end;

procedure TRsColorPicker.SetSelectedColor(const Value: TColor);
begin
  if Value <> FPickerForm.SelectedColor then
    begin
      FPickerForm.SelectedColor := Value;
      Invalidate;
    end;
end;

procedure TRsColorPicker.WMContextMenu(var Message: TWMContextMenu);
begin
  SetFocus;
  if PopupMenu <> nil then
    inherited;
end;

procedure TRsColorPicker.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Invalidate;
  with FPickerForm do
    if not Focused and Timer1.Enabled then
      Hide;
end;

procedure TRsColorPicker.WMPaint(var Msg: TWMPaint);
var
  R            : TRect;
  PS           : TPaintStruct;
  DC           : HDC;
  OldPenStyle  : TPenStyle;
  OldBrushStyle: TBrushStyle;
begin
  DC := Msg.DC;
  if Msg.DC = 0 then
    DC := BeginPaint(Handle, PS);

  try
    FCanvas.Handle := DC;
    try
      FCanvas.Font := Font;
      FCanvas.Brush := Brush;
      Winapi.Windows.GetClientRect(Handle, R);

      with FColorButton do
        if Align = alRight then
          Dec(R.Right, Width)
        else if FColorButton.Align = alLeft then
          Inc(R.Left, Width);

      with FCanvas do
        begin
          Brush.Color := Color;
          Winapi.Windows.FillRect(DC, R, Brush.Handle);
          InflateRect(R, -2, -2);

          if SelectedColor = clNone then
            begin
              OldPenStyle := Pen.Style;
              OldBrushStyle := Brush.Style;

              if Focused  then
                begin
                  Pen.Color := clBtnShadow;
                  Brush.Color := clBtnShadow;
                end
              else
                begin
                  Pen.Color := clBtnFace;
                  Brush.Color := clBtnFace;
                end;

              Brush.Style := bsBDiagonal;
              Rectangle(R);
              Brush.Style := OldBrushStyle;
              Pen.Style := OldPenStyle;
            end
          else
            begin
              DrawEdge(DC, R, BDR_RAISEDOUTER, BF_RECT or BF_ADJUST or BF_FLAT);
              Brush.Color := SelectedColor;
              Winapi.Windows.FillRect(DC, R, Brush.Handle);
            end;
        end;

      if Focused then
        DrawFocusRect(DC, R);
    finally
      FCanvas.Handle := 0;
    end;
  finally
    if Msg.DC = 0 then
      ReleaseDC(Handle, DC);
  end;
end;

procedure TRsColorPicker.WMSetCursor(var Msg: TWMSetCursor);
begin
  FColorButton.Cursor := Cursor;
  if Cursor <> crDefault then
    inherited;
end;

procedure TRsColorPicker.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  HideCaret(Handle);
  Invalidate;
end;


end.
