unit RsPickerForm;

{
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in compliance
  with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/MPL-1.1.html

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
  the specific language governing rights and limitations under the License.

  Copyright (c) 2012  .. (UA3GDW)
  E-mail: skl@griazy.lipetsk.ru, ua3gdw@radiosoft.info
  URL: http://radiosoft.info
}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, VCL.Graphics, VCL.Controls, VCL.Forms, VCL.Dialogs,
  VCL.ExtCtrls, RsButtons, Vcl.Buttons;

type
  TPickerForm = class(TForm)
    ColorDialog1: TColorDialog;
    RsSpeedButton1: TRsSpeedButton;
    RsSpeedButton2: TRsSpeedButton;
    Timer1: TTimer;
    procedure RsSpeedButton1Click(Sender: TObject);
    procedure RsSpeedButton2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Shape2MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ColorDialog1Show(Sender: TObject);
    procedure ColorDialog1Close(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FParentForm: TWinControl;
    FSelectedColor: TColor;
    FIsShow: Boolean;
    FOnColorSelect: TNotifyEvent;

    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure ClickColorBottomEvents(Sender: TObject);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    property SelectedColor: TColor read FSelectedColor write FSelectedColor;
    procedure Hide;
    procedure CreateColors;
    property ParentForm: TWinControl read FParentForm write FParentForm;
    property IsShow: Boolean read FIsShow write FIsShow;
    property OnColorSelect: TNotifyEvent read FOnColorSelect write FOnColorSelect;
  end;

implementation

{$R *.dfm}

uses RsSTDRes;

{ TPickerForm }

procedure TPickerForm.ClickColorBottomEvents(Sender: TObject);
begin
  FSelectedColor := TColor(TRsSpeedButton(Sender).Tag);
  if Assigned(FOnColorSelect) then
    FOnColorSelect(Self);
  Hide;
end;

procedure TPickerForm.ColorDialog1Close(Sender: TObject);
begin
  Application.ModalFinished;
end;

procedure TPickerForm.ColorDialog1Show(Sender: TObject);
begin
  Application.ModalStarted;
end;

procedure TPickerForm.CreateColors;

  procedure CreateColorButtons(const ARow, Offset: Integer);
  const
    CLRVAL: array [0..39] of TColor = ($00000000, $00003399, $00003333,
      $00003300, $00663300, $00800000, $00993333, $00333333, $00000080,
      $000066FF, $00008080, $00008000, $00808000, $00FF0000, $00996666,
      $00808080, $000000FF, $000099FF, $0000CC99, $00669933, $00999933,
      $00FF6633, $00800080, $00999999, $00FF00FF, $0000CCFF, $0000FFFF,
      $0000FF00, $00FFFF00, $00F0CAA6, $00663399, $00C0C0C0, $00CC99FF,
      $0099CCFF, $0099FFFF, $00CCFFCC, $00FFFFCC, $00FFCC99, $00FF99CC,
      $00FFFFFF);
  var
    i, iH: Integer;
    Btn  : TRsSpeedButton;
    Bmp  : TBitmap;
  begin
    if RsSpeedButton2.Visible then
      begin
        iH := 28;
        RsSpeedButton1.Top := 121;
        Height := 148;
      end
    else
      begin
        RsSpeedButton1.Top := 97;
        Height := 124;
        iH := 5;
      end;

    Bmp := TBitmap.Create;
    try
      with Bmp do
        begin
          Width := 14;
          Height := 14;
        end;

      for i := (ARow * 7) + Offset to ((ARow + 1) * 7) + Offset do
        begin
          Btn := TRsSpeedButton.Create(Self);
          with Btn do
            begin
              with Bmp.Canvas do
                begin
                  Brush.Style := bsSolid;
                  Brush.Color := $00000001;
                  FillRect(Rect(0, 0, 14, 14));
                  Brush.Color := CLRVAL[i];
                  FillRect(Rect(2, 2, 12, 12));
                  Brush.Color := clGray;
                  FrameRect(Rect(1, 1, 13, 13));
                end;

              Width := 18;
              Height := 18;
              Left := 18 * (i - ((ARow * 7) + Offset)) + 5;
              Top := iH + (18 * ARow);
              Margin := 2;
              Flat := True;
              Tag := CLRVAL[i];
              Bitmap.Assign(Bmp);
              OnClick := ClickColorBottomEvents;
              Parent := Self;
            end;
        end;
    finally
      Bmp.Free;
    end;
  end;

var
  i: Integer;
begin
  for i := 0 to 4 do
    CreateColorButtons(i, i);
end;

procedure TPickerForm.CreateParams(var Params: TCreateParams);
begin
  inherited;

  with Params do
    begin
      Style := Style or WS_POPUP or WS_BORDER;
      WindowClass.Style := WindowClass.Style or CS_SAVEBITS;
      if NewStyleControls then
        ExStyle := WS_EX_TOOLWINDOW;

      if CheckWin32Version(5, 1) then
        WindowClass.Style := WindowClass.style or CS_DROPSHADOW;
    end;
end;

procedure TPickerForm.FormCreate(Sender: TObject);
begin
  RsSpeedButton1.Caption := RES_101;
  RsSpeedButton2.Caption := RES_100;
end;

procedure TPickerForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Timer1.Enabled := False;
  Hide;
end;

procedure TPickerForm.Hide;
begin
  MouseCapture := False;
  Timer1.Enabled := False;
  SetWindowPos(Handle, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE or
    SWP_NOMOVE or SWP_HIDEWINDOW);

  FIsShow := False;
  if (FParentForm <> nil) and FParentForm.Visible then
    FParentForm.SetFocus;

  TCustomControl(Owner).Invalidate;
end;

procedure TPickerForm.RsSpeedButton1Click(Sender: TObject);
begin
  Hide;
  with ColorDialog1 do
    begin
      Color := FSelectedColor;

      if Execute then
        begin
          FSelectedColor := Color;
          TCustomControl(Self.Owner).Invalidate;
          if Assigned(FOnColorSelect) then
            FOnColorSelect(Self);
        end;
    end;
end;

procedure TPickerForm.RsSpeedButton2Click(Sender: TObject);
begin
  FSelectedColor := clNone;
  if Assigned(FOnColorSelect) then
    FOnColorSelect(Self);
  Hide;
end;

procedure TPickerForm.Shape2MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Hide;
  SetWindowPos(Handle, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE or
    SWP_NOACTIVATE or SWP_SHOWWINDOW);
  Timer1.Enabled := True;
end;

procedure TPickerForm.Timer1Timer(Sender: TObject);
var
  R: TRect;
  P: TPoint;
begin
  GetCursorPos(P);
  GetWindowRect(Handle, R);
  if not PtInRect(R, P) then
    MouseCapture := True
  else
    MouseCapture := False;
end;

procedure TPickerForm.WMNCHitTest(var Message: TWMNCHitTest);
begin
  Message.Result := HTTRANSPARENT;
end;

end.
