unit RsReg;

{
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in compliance
  with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/MPL-1.1.html

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
  the specific language governing rights and limitations under the License.

  Copyright (c) 2012  .. (UA3GDW)
  E-mail: skl@griazy.lipetsk.ru, ua3gdw@radiosoft.info
  URL: http://radiosoft.info
}

interface

uses Winapi.Windows, System.SysUtils, System.Classes, VCL.Graphics, VCL.ImgList,
  DesignIntf, DesignEditors, VCLEditors, RsButtons, RsColorPicker;

type
  TImageIndexProperty = class(TIntegerProperty, ICustomPropertyListDrawing)
  protected
    function GetImages: TCustomImageList; virtual;
    property Images: TCustomImageList read GetImages;
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure ListMeasureWidth(const Value: String; ACanvas: TCanvas;
      var AWidth: Integer);
    procedure ListMeasureHeight(const Value: String; ACanvas: TCanvas;
      var AHeight: Integer);
    procedure ListDrawValue(const Value: String; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean);
  end;

procedure Register;

implementation

uses TypInfo;

procedure Register;
begin
  RegisterComponents('RsDesign STD', [TRsBitBtn, TRsSpeedButton,
    TRsColorPicker]);
  RegisterPropertyEditor(TypeInfo(TImageIndex), TRsBitBtn, 'ImageIndex',
    TImageIndexProperty);
  RegisterPropertyEditor(TypeInfo(TImageIndex), TRsSpeedButton, 'ImageIndex',
    TImageIndexProperty);
end;

{ TImageIndexProperty  }

function TImageIndexProperty.GetImages: TCustomImageList;
begin
  Result := GetObjectProp(GetComponent(0), 'Images') as TCustomImageList;
end;

function TImageIndexProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paValueList];
end;

procedure TImageIndexProperty.GetValues(Proc: TGetStrProc);
var
  i: Integer;
begin
  Proc('-1');
  if Images = nil then
    Exit;

  for i := 0 to Images.Count - 1 do
    Proc(IntToStr(I));
end;

procedure TImageIndexProperty.ListMeasureWidth(const Value: String;
  ACanvas: TCanvas; var AWidth: Integer);
begin
  inherited;
  if Images <> nil then
    Inc(AWidth, 1 + Images.Width + 1);
end;

procedure TImageIndexProperty.ListMeasureHeight(const Value: String;
  ACanvas: TCanvas; var AHeight: Integer);
begin
  inherited;
  if (Images <> nil) and (AHeight < 1 + Images.Height + 1) then
    AHeight := 1 + Images.Height + 1;
end;

procedure TImageIndexProperty.ListDrawValue(const Value: String; ACanvas: TCanvas;
  const ARect: TRect; ASelected: Boolean);
var
  ATextRect     : TRect;
  PrevBrushColor: TColor;
begin
  ATextRect := ARect;

  if Images <> nil then
    begin
      Inc(ATextRect.Left, 1 + Images.Width + 1);
      with ARect do
        begin
          PrevBrushColor := ACanvas.Brush.Color;
          ACanvas.Brush.Color := clWindow;
          ACanvas.FillRect(Rect(Left, Top, ATextRect.Left, Bottom));
          ACanvas.Brush.Color := PrevBrushColor;
        end;
      Images.Draw(ACanvas, ARect.Left + 1, ARect.Top + 1, StrToInt(Value));
    end;

  with ATextRect, ACanvas do
    ACanvas.TextRect(ATextRect, Left + 1,
      (Top + Bottom - TextHeight(Value)) div 2, Value);
end;

end.
